



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
cython --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
cython --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
cythonize --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
cygdb --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
